--[[
  ****************************************************************
	GLock
	
	A Suite of Warlock Utilities including a Mob resistance calculator
	v2.5 - Added Eye of Moam Support, GLOCKLOCK function to lock UI, and set up for possibal localization
	
	v2.0 - Added configuration menu and an ability to search the database

	v1.3 - Added wand support.  Auto-chat channel joining.  Cleaner UI.  Minimize UI mode
	
	v1.2 - Added common chat channel for multi-user database contribution.  More bug fixes and tweaks to the calculation algorithm
	
	v1.1 - First Overhaul.  Added school detection for total resists, cleaner calculations, more data per mob and Binary Frost resist calculations
	
	v1.0 - Initial Release.  Only smart raid curse and resist calculator available

	Written by Ogdin of the Deathreaver Legion.  Shadow Moon
	Contact via posts at http://www.atlantic-orcs.org/

-- TODO
-- AUTO-detect leaving sync channel and clear GLOCK_Config["CHATCHANNEL"]

	
	****************************************************************]]


function GLOCK_version()

GLOCK_Config["VER"]="3.0"
DEFAULT_CHAT_FRAME:AddMessage("GLOCK version "..GLOCK_Config["VER"].." Loaded (type /glockhelp for a list of commands)")

end

function GLOCK_test(a)
--/gltest  --I put this here to test arbitrary code from in game as i'm learning

end

function GLOCK_chatStatus()

--This is the OnUpdate event.. The contents of this if statement are executed every 1 second.
		if (GLOCK_TIME<math.floor(GetTime())) then 
			GMaxActivity:SetVertexColor(1,0,0);
			GMinActivity:SetVertexColor(1,0,0);
		end
		GLOCK_TIME=math.floor(GetTime());
end


function GLOCK_help()

DEFAULT_CHAT_FRAME:AddMessage("/gljoin <channel> - links GLOCK to a common channel")
DEFAULT_CHAT_FRAME:AddMessage("/glocklock - toggles locking the GLOCK UI in place")
DEFAULT_CHAT_FRAME:AddMessage("/glock - bring up the config/search panel")
DEFAULT_CHAT_FRAME:AddMessage("/glraidcurse - Smart Raid Curse script")

end


function GLOCK_lock()
--toggles lock status
	if GLOCK_frame:GetScript("OnMouseDown")==nil then
		GLOCK_frame:SetScript("OnMouseDown",glocklockmax)
		GLOCK_minimize:SetScript("OnMouseDown",glocklockmin)
		DEFAULT_CHAT_FRAME:AddMessage("GLOCK Unlocked.")
		GLOCK_Config["LOCKED"]=0
	else
		GLOCK_frame:SetScript("OnMouseDown",nil)
		GLOCK_minimize:SetScript("OnMouseDown",nil)
		DEFAULT_CHAT_FRAME:AddMessage("GLOCK locked.")
		GLOCK_Config["LOCKED"]=1
	end

end

function glocklockmax() GLOCK_frame:StartMoving() end
function glocklockmin() GLOCK_minimize:StartMoving() end

function glconfig()

GLOCK_config:Show()

end


----------------------
-- Event Handler
-- this function parses events
function GLOCK_OnEvent()
	
	if event=="CHAT_MSG_SPELL_SELF_DAMAGE" then GLOCK_DmgParse() end
	if event=="UNIT_INVENTORY_CHANGED" then GLOCK_scanGear() end
	if event=="UNIT_AURA" then GLOCK_scanTarget() end
	if event=="UNIT_AURASTATE" then GLOCK_scanTarget() end
	if event=="PLAYER_TARGET_CHANGED" then GLOCK_scanTarget() end
	if event=="CHAT_MSG_CHANNEL" then GLOCK_chatHandle() end
	if event=="CHAT_MSG_RAID" then GLOCK_syncChanRaid() end
	if event=="VARIABLES_LOADED" then GLOCK_Initialize() end

end



-----------------------------------------------------------------------------------------------------------
--This is the main function that handles resist calculation for your damage data
-----------------------------------------------------------------------------------------------------------
function GLOCK_ResistCalc()


--Correct for rounding error
	if (gmitigated>10 and gmitigated<=35) then gmitigated=25 end
	if (gmitigated>35 and gmitigated<=60) then gmitigated=50 end
	if (gmitigated>60 and gmitigated<=85) then gmitigated=75 end
	
	if not MobResistDB[GLOCK_dbindex] then MobResistDB[GLOCK_dbindex]="0/0_0A0|0"; end

--EXTRACT DATABASE ENTRY
	a=MobResistDB[GLOCK_dbindex];
	GLOCK_temp1=string.find(a,"/");
	GLOCK_temp2=string.find(a,"_");
	GLOCK_temp3=string.find(a,"A");
	GLOCK_temp4=string.find(a,"|");

	GLOCK_DBMiti=tonumber(string.sub(a,1,GLOCK_temp1-1));				--Current Resist Approximation
	GLOCK_DBHits=tonumber(string.sub(a,GLOCK_temp1+1,GLOCK_temp2-1))		--#of spells that have hit
	GLOCK_DBMiss=tonumber(string.sub(a,GLOCK_temp2+1,GLOCK_temp3-1))		--#of 100% resists
	GLOCK_DBMinR=tonumber(string.sub(a,GLOCK_temp3+1,GLOCK_temp4-1))		--Minimum spell penetration that a partial resist has been observed at
	GLOCK_DBi=tonumber(string.sub(a,GLOCK_temp4+1))					--Hit number for first partial resist to start learning

--------------------------------------------------------RESIST CALCULATION STARTS HERE------------------------------------------------
	
--IF SPELL TOTALLY RESISTED
	if gmitigated==100 then 
		--MISS INCREMENT and track average penetration value
		
		GLOCK_DBMiss=GLOCK_DBMiss+1;
		if GLOCK_cast_school==GLOCK_schools[1] then GLOCK_temp1=GLElemPrec*2 else GLOCK_temp1=0 end
		GLOCK_DBi=(GLOCK_DBi*(GLOCK_DBHits+GLOCK_DBMiss-1)+(GLhit-GLOCK_temp1))/(GLOCK_DBHits+GLOCK_DBMiss)


		if (GLOCK_cast_school==GLOCK_schools[3]) then    --100% resists are only relevant in frost calculations currently
			GLHITS=GLOCK_DBMiss+GLOCK_DBHits;
			GLOCK_DBMiti=100-((100-GLOCK_DBMiti)*(GLHITS-1)+(0-gpenetration)/GLOCK_levelMiss(GLOCK_Target_Level))/GLHITS
			GLOCK_DBMinR=GLOCK_DBMinR-(GLOCK_DBMinR-gpenetration*4)*GLOCK_factor
		end

		MobResistDB[GLOCK_dbindex]=string.format("%f/%f_%fA%f|%f",GLOCK_DBMiti,GLOCK_DBHits,GLOCK_DBMiss,GLOCK_DBMinR,GLOCK_DBi);
		if (GLOCK_Config["CHATCHANNEL"]) then
			if GLOCK_cast_school==GLOCK_schools[1] then GLOCK_temp1=GLElemPrec*2 else GLOCK_temp1=0 end
			SendChatMessage("<GLOCK3>"..GLOCK_dbindex..",,"..tostring(100).."++"..gpenetration.."=="..(0).."__"..(GLhit-GLOCK_temp1),"CHANNEL",nil,GetChannelName(GLOCK_Config["CHATCHANNEL"]))
		end
		
		GLOCK_UpdateGUI()
		return; 
	end

	


--Hit increment
	GLOCK_DBHits=GLOCK_DBHits+1;

	


--GLOCK_factor adapts as you get closer to the target (i.e. more samples available)
	if (GLOCK_DBHits)>GLCUSPS[1] then GLOCK_factor=GLFACTORS[1] end
	if (GLOCK_DBHits)>GLCUSPS[2] then GLOCK_factor=GLFACTORS[2] end
	if (GLOCK_DBHits)>GLCUSPS[3] then GLOCK_factor=GLFACTORS[3] end
	if (GLOCK_DBHits)>GLCUSPS[4] then GLOCK_factor=GLFACTORS[4] end
	if (GLOCK_DBHits)>GLCUSPS[5] then GLOCK_factor=GLFACTORS[5] end
	if (GLOCK_DBHits)>GLCUSPS[6] then GLOCK_factor=GLFACTORS[6] end
	if (GLOCK_DBHits)>GLCUSPS[7] then GLOCK_factor=GLFACTORS[7] end
	if (GLOCK_DBHits)>GLCUSPS[8] then GLOCK_factor=GLFACTORS[8] end
	
--Track Average +hit% on this mob for this school (ignore elemental precision if school is arcane)
	if GLOCK_cast_school==GLOCK_schools[1] then GLOCK_temp1=GLElemPrec*2 else GLOCK_temp1=0 end
	GLOCK_DBi=(GLOCK_DBi*(GLOCK_DBHits+GLOCK_DBMiss-1)+(GLhit-GLOCK_temp1))/(GLOCK_DBHits+GLOCK_DBMiss)
--Track Average penetration on this mob for this school
	GLOCK_DBMinR=GLOCK_DBMinR-(GLOCK_DBMinR-gpenetration*4)*GLOCK_factor

--CALCULATE THE NEW RESIST VALUE
	if GLOCK_cast_school~=GLOCK_schools[3] then
		GLOCK_diff=(gmitigated+gpenetration-GLOCK_DBMiti)*GLOCK_factor;
		GLOCK_DBMiti=GLOCK_DBMiti + GLOCK_diff;
	end

--SPECIFIC CALCULATION FOR BINARY FROST SPELLS (this also works with partially resisted frost spells such as frost wand damage)
	if GLOCK_cast_school==GLOCK_schools[3] then
		GLHITS=GLOCK_DBMiss+GLOCK_DBHits;
		GLOCK_DBMiti=100-((100-GLOCK_DBMiti)*(GLHITS-1)+(100-gmitigated-gpenetration)/GLOCK_levelMiss(GLOCK_Target_Level))/GLHITS
	end

--------------------------------------------------------RESIST CALCULATION ENDS HERE------------------------------------------------

--STORE DATA
	MobResistDB[GLOCK_dbindex]=string.format("%f/%f_%fA%f|%f",GLOCK_DBMiti,GLOCK_DBHits,GLOCK_DBMiss,GLOCK_DBMinR,GLOCK_DBi);

	
	if GLOCK_Config["CHATCHANNEL"] then
		if GLOCK_cast_school==GLOCK_schools[1] then GLOCK_temp1=GLElemPrec*2 else GLOCK_temp1=0 end
		SendChatMessage("<GLOCK3>"..GLOCK_dbindex..",,"..gmitigated.."++"..gpenetration.."=="..(0).."__"..(GLhit-GLOCK_temp1),"CHANNEL",nil,GetChannelName(GLOCK_Config["CHATCHANNEL"]))
	end

	GLOCK_UpdateGUI()
	

end

-----------------------------------------------------------------------------------------------------------
--Determines your hit chance on the target.  As per Tseric's post and assuming 99% max hit chance
-----------------------------------------------------------------------------------------------------------
function GLOCK_levelMiss(GLEVEL)

	TempMiss=99;
	if GLEVEL==UnitLevel("Player") then TempMiss=96;
	elseif GLEVEL==(UnitLevel("Player")+1) then TempMiss=95;
	elseif GLEVEL==(UnitLevel("Player")+2) then TempMiss=94; 
	elseif GLEVEL==(UnitLevel("Player")+3) then TempMiss=83;
	elseif GLEVEL==(UnitLevel("Player")+4) then TempMiss=72;
	elseif GLEVEL==(-1) then TempMiss=83;   --Made the assumption that bosses will be equated to level 63 mobs
	end

	TempMiss=TempMiss+GLhit;
	if TempMiss>99 then TempMiss=99 end

	--base resist value is 1%, multiply TempMiss by .99 to account for this
	return TempMiss/100

end

-----------------------------------------------------------------------------------------------------------
--Handles graphics updates for the data on screen
-----------------------------------------------------------------------------------------------------------
function GLOCK_UpdateGUI()

	if GLOCK_Config["VISIBLE"]==0 then GLOCK_frame:Hide() GLOCK_minimize:Hide() return end
	if GLOCK_Target_Name==nil then return end
	GLOCK_frame:Hide()
	GLOCK_minimize:Hide()
	jj=1;

--COLOR LEVEL DIFFERENCE
	GLOCK_Tlvl:SetText(GLOCK_Target_Level)
	GLOCK_TName:SetText(GLOCK_Target_Name)
	if GLOCK_Target_Level>=0 then GLOCK_Tlvl:SetTextColor(.5,.5,.5) end
	if GLOCK_Target_Level>=48 then GLOCK_Tlvl:SetTextColor(.2,.9,.2) end
	if (GLOCK_Target_Level>=58 and GLOCK_Target_Level<=62) then GLOCK_Tlvl:SetTextColor(1,1,0) end
	if (GLOCK_Target_Level==63) then GLOCK_Tlvl:SetTextColor(.9,.5,.4) end
	if (GLOCK_Target_Level>63) then GLOCK_Tlvl:SetTextColor(1,.2,.2) end
	if (GLOCK_Target_Level==-1) then GLOCK_Tlvl:SetTextColor(1,.2,.2); GLOCK_Tlvl:SetText("Boss") end


	for i=1,5 do
		GLOCK_dbindex=GLOCK_Target_Name.."/"..GLOCK_Target_Level.."/"..GLOCK_schools[i]
		getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):Hide()

		if (MobResistDB[GLOCK_dbindex.."immune"]==1 or MobResistDB[GLOCK_dbindex.."vuln"]==1) then 
			if MobResistDB[GLOCK_dbindex.."immune"]==1 then GLSpecial="Imm" end
			if MobResistDB[GLOCK_dbindex.."vuln"]==1 then GLSpecial="Vul" end
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetText(GLSpecial) 
			getglobal("GMin"..GLOCK_UIschools[i].."TxT"):SetText(GLSpecial)
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetText("0")
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetText("0")

			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):Show()

			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):SetPoint("TOPLEFT",5,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):SetPoint("TOPLEFT",25,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetPoint("TOPLEFT",70,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetPoint("TOPLEFT",110,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetPoint("TOPLEFT",160,-45-10*(jj-1));
			jj=jj+1;			
		end

		if (MobResistDB[GLOCK_dbindex]~=nil and (not(MobResistDB[GLOCK_dbindex.."immune"]==1 or MobResistDB[GLOCK_dbindex.."vuln"]==1))) then 
			a=MobResistDB[GLOCK_dbindex];
			GLOCK_temp1=string.find(a,"/");
			GLOCK_temp2=string.find(a,"_");
			GLOCK_temp3=string.find(a,"A");
			GLOCK_temp4=string.find(a,"|");
			
			GLOCK_DBMiti=tonumber(string.sub(a,1,GLOCK_temp1-1));			
			GLOCK_DBHits=tonumber(string.sub(a,GLOCK_temp1+1,GLOCK_temp2-1))
			GLOCK_DBMiss=tonumber(string.sub(a,GLOCK_temp2+1,GLOCK_temp3-1))	
			GLOCK_DBMinR=tonumber(string.sub(a,GLOCK_temp3+1,GLOCK_temp4-1))	
			GLOCK_DBi=tonumber(string.sub(a,GLOCK_temp4+1))				
			
		--CHECK FOR ERRORS
			if GLOCK_DBMiti==nil then GLOCK_DBMiti=0; MobResistDB[GLOCK_dbindex]=nil; return end
			if GLOCK_DBHits==nil then GLOCK_DBHits=0; MobResistDB[GLOCK_dbindex]=nil; return end
			if GLOCK_DBMiss==nil then GLOCK_DBMiss=0; MobResistDB[GLOCK_dbindex]=nil; return end

			if GLOCK_DBMiti<0 then GLOCK_DBMiti=0; end    --Sometimes Frost resist scores appear negative in the calculation

		--Update Resist Approximations
			GTxT=tostring(GLOCK_round(4*GLOCK_DBMiti));
			getglobal("GMin"..GLOCK_UIschools[i].."TxT"):SetText(GTxT)
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetText(GTxT)
			
		--Display #Casts
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetText(GLOCK_round(GLOCK_DBHits+GLOCK_DBMiss))

		--Display %Hit (avg +hit%)  (i.e. 86(3)% )
			GTxT=string.format("%.0f(%.0f)",GLOCK_round(100*GLOCK_DBHits/(GLOCK_DBMiss+GLOCK_DBHits)),GLOCK_DBi)
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetText(GTxT)

		--Display Current Penetration (avg Penetration) (i.e. 75(60) )
			GTxT=string.format("%.0f(%.0f)",getglobal("GLD"..string.lower(GLOCK_UIschools[i]))+GLnegresist,GLOCK_DBMinR)
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):SetText(GTxT)


			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):Show()

			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):SetPoint("TOPLEFT",5,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):SetPoint("TOPLEFT",25,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetPoint("TOPLEFT",70,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetPoint("TOPLEFT",110,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetPoint("TOPLEFT",160,-45-10*(jj-1));
			jj=jj+1;

		elseif ((not MobResistDB[GLOCK_dbindex]) and (not MobResistDB[GLOCK_dbindex.."immune"]) and (not MobResistDB[GLOCK_dbindex.."vuln"])) then
			getglobal("GMin"..GLOCK_UIschools[i].."TxT"):SetText("")
		end
	end

	
	if MobResistDB["UISTATE"]=="MAXIMIZED" then
		GLOCK_frame:Show()
		GLOCK_minimize:Hide()
	else
		GLOCK_frame:Hide()
		GLOCK_minimize:Show()
	end

end


-----------------------------------------------------------------------------------------------------------
--Determine the debuffs on the target or buffs on the player that add to penetration
-----------------------------------------------------------------------------------------------------------
function GLOCK_scanTarget()

	GLDshadow=0;
	GLDarcane=0;
	GLDfrost=0;
	GLDfire=0;
	GLDnature=0;

	
	--if unit is a friendly NPC, then hide glock
	if ((UnitIsFriend("Target","Player")) and (not UnitIsPlayer("Target"))) then GLOCK_frame:Hide(); GLOCK_minimize:Hide() return; end
	
	--If the unit is friendly PC and pvp is off, then hide glock
	if ((not UnitIsEnemy("Target","Player")) and (not GLOCK_Config["PVP"]) and UnitIsPlayer("Target")) then GLOCK_frame:Hide(); GLOCK_minimize:Hide() return; end

	--If PvP is off and the target is an enemy player, then hide glock		
	if ( not GLOCK_Config["PVP"] )  then
		if (UnitIsPlayer("Target") and UnitIsEnemy("Target","Player")) then GLOCK_frame:Hide(); GLOCK_minimize:Hide() return; end
	end

--ScanTarget for Resist debuffs


	GLOCK_Target_Name=UnitName("Target");
	if UnitIsPlayer("Target") then GLOCK_Target_Name="*"..UnitName("Target").."*" end
	GLOCK_Target_Level=UnitLevel("Target");
	GLOCK_YourHitT:SetTextColor(1,1,0)
	
	if not GLOCK_Target_Name then 
		GLOCK_frame:Hide();
		GLOCK_minimize:Hide()
		GLOCK_confirm:Hide()
		for i=1,5 do getglobal("GLOCK_confirm_"..GLOCK_UIschools[i]):SetChecked(1) end
		return;
	end
		
		TFury=0;

--DEBUFF SCANNING
	for i=1,16 do
		GLTooltip:ClearLines();	
		GLTooltip:SetUnitDebuff("Target",i);
		if (GLTooltipTextLeft1:GetText()==GL_COS) then GLDshadow=GLDshadow+75; GLDarcane=GLDarcane+75 end
		if (GLTooltipTextLeft1:GetText()==GL_COE) then GLDfrost=GLDfrost+75; GLDfire=GLDfire+75 end
		if (GLTooltipTextLeft1:GetText()==GL_TF and TFury==0) then GLDnature=GLDnature+25; TFury=1; end
	end


--EYE OF MOAM BUFF ON PLAYER
	for i=1,16 do
		GLTooltip:ClearLines();
		GLTooltip:SetUnitBuff("Player",i);
		if GLTooltipTextLeft1:GetText()==GL_Eye then 
			GLDshadow=GLDshadow+100
			GLDarcane=GLDarcane+100
			GLDfrost=GLDfrost+100
			GLDfire=GLDfire+100
			GLDnature=GLDnature+100
		end
	end

	GLOCK_UpdateGUI()


end

-----------------------------------------------------------------------------------------------------------
--Keep track of penetration and hit chance from gear you have equipped
-----------------------------------------------------------------------------------------------------------
function GLOCK_scanGear()

--DETERMINE HOW MUCH -RESIST and +HIT gear player has on

	GLnegresist=0;
	GLhit=0;
	GLARCANISTSet=0;
	GLZANZILSet=0

	for i=1,18 do
		hasItem = GLTooltip:SetInventoryItem("player", i);
		if ( not hasItem ) then
			GLTooltip:ClearLines()
		else
			itemName = GLTooltipTextLeft1:GetText();

			lines = GLTooltip:NumLines();

			for i=2, lines do
				tmpText = getglobal("GLTooltipTextLeft"..i);
				if (tmpText:GetText()) then
					tmpStr = tmpText:GetText();

					if string.find(tmpStr,GLEQUIP_PENETRATION) then
						tmpStr=string.gsub(tmpStr,GLEQUIP_PENETRATION,"")
						tmpStr=string.sub(tmpStr,1,-2)
						GLnegresist=GLnegresist+tonumber(tmpStr);
					end
					if string.find(tmpStr,GLEQUIP_HIT) then
						tmpStr=string.gsub(tmpStr,GLEQUIP_HIT,"")
						tmpStr=string.sub(tmpStr,1,1)
						GLhit=GLhit+tonumber(tmpStr);
					elseif string.find(tmpStr,GLMAGE_ZGHIT) then    ---Mage helm/leg ZG enchant
						GLhit=GLhit+1;
					elseif string.find(tmpStr,GLARCANIST) then
						GLARCANISTSet=GLARCANISTSet+1;
					elseif string.find(tmpStr,GLZANZIL) then
						GLZANZILSet=GLZANZILSet+1;
					end
				end
			end
		end
	end	
		if GLARCANISTSet>=5 then GLnegresist=GLnegresist+10; end
		if GLZANZILSet==2 then GLhit=GLhit+1 end

--This Elemental Precision and Arcane Subtlety talent scan was written by Aelian of Theorycraft
	local _, class = UnitClass("player")
	if string.lower(class) == GLMAGE then
		_, _, _, _, GLElemPrec = GetTalentInfo(3, 3)
		GLhit = GLhit + GLElemPrec*2
		local _, _, _, _, currank = GetTalentInfo(1, 1)
		GLnegresist = GLnegresist + currank*5
	end
	

	if ((not GLDshadow) or (not GLDarcane) or (not GLDfrost) or (not GLDfire) or (not GLDnature)) then GLOCK_scanTarget() end

	GLOCK_HitP:SetText(GLhit.."% Hit")
	GLOCK_ShadowVuln:SetText(GLDshadow+GLnegresist)
	GLOCK_ArcaneVuln:SetText(GLDarcane+GLnegresist)
	GLOCK_FrostVuln:SetText(GLDfrost+GLnegresist)
	GLOCK_FireVuln:SetText(GLDfire+GLnegresist)
	GLOCK_NatureVuln:SetText(GLDnature+GLnegresist)
	GLOCK_scanTarget()

end

function GLOCK_ClearEntry()

	for i=1,5 do
		if getglobal("GLOCK_confirm_"..GLOCK_UIschools[i]):GetChecked() then
			GLOCK_dbindex=GLOCK_Target_Name.."/"..GLOCK_Target_Level.."/"..GLOCK_schools[i]
			MobResistDB[GLOCK_dbindex]=nil
			MobResistDB[GLOCK_dbindex.."immune"]=nil
			MobResistDB[GLOCK_dbindex.."vuln"]=nil
			getglobal("GLOCK_confirm_"..GLOCK_UIschools[i]):SetChecked(1)
		end
	end

	GLOCK_UpdateGUI()
	GLOCK_confirm:Hide()

end

-----------------------------------------------------------------------------------------------------------
--Catch Chat events in the GLOCK link channel and parse out the data for calculation
-----------------------------------------------------------------------------------------------------------
function GLOCK_chatHandle()
	if not string.find(arg4,GLOCK_Config["CHATCHANNEL"]) then return end
	if not string.find(arg1,"<GLOCK3>") then return end
	if arg2==UnitName("Player") then return end


--Set Indicators Green	
	GMaxActivity:SetVertexColor(0,1,0);
	GMinActivity:SetVertexColor(0,1,0);

	tmpStr=string.gsub(arg1,"<GLOCK3>","");
	
	--Parse it
	GLOCK_temp1=string.find(tmpStr,",,")
	GLOCK_temp2=string.find(tmpStr,"++")
	GLOCK_temp3=string.find(tmpStr,"==")
	GLOCK_temp4=string.find(tmpStr,"__")
	if (GLOCK_temp1==nil or GLOCK_temp2==nil or GLOCK_temp3==nil or GLOCK_temp4==nil) then return end
	

	GLOCK_sync_dbindex=string.sub(tmpStr,1,GLOCK_temp1-1);

	Ga=string.find(tmpStr,"/");
	Gb=string.find(tmpStr,"/",Ga+1);
	Gc=string.find(tmpStr,"/",Gb+1);

	GLOCK_sync_Name=string.sub(tmpStr,1,Ga-1);
	GLOCK_sync_Level=tonumber(string.sub(tmpStr,Ga+1,Gb-1))
	GLOCK_sync_school=string.sub(tmpStr,Gb+1,GLOCK_temp1-1)


	GLOCK_sync_gmitigated=tonumber(string.sub(tmpStr,GLOCK_temp1+2,GLOCK_temp2-1))
	GLOCK_sync_gpenetration=tonumber(string.sub(tmpStr,GLOCK_temp2+2,GLOCK_temp3-1))
	GLOCK_sync_DBi=tonumber(string.sub(tmpStr,GLOCK_temp3+2,GLOCK_temp4-1))
	GLOCK_sync_GLhit=tonumber(string.sub(tmpStr,GLOCK_temp4+2))

	
	if GLOCK_sync_gmitigated==nil then 
		if UnitName("Player")=="Ogdin" then DEFAULT_CHAT_FRAME:AddMessage(arg1) end
	return end

--VERBOSE CHAT IN CONFIG PANNEL
		GLOCK_Config["VCHAT1"]=GLOCK_Config["VCHAT2"]
		GLOCK_Config["VCHAT2"]=GLOCK_Config["VCHAT3"]
		GLOCK_Config["VCHAT3"]=GLOCK_Config["VCHAT4"]
		GLOCK_Config["VCHAT4"]="<"..date()..">  "..arg2.." - "..GLOCK_sync_Name.." - "..GLOCK_sync_school.." - "..GLOCK_sync_gmitigated.."%".." @ -"..(GLOCK_sync_gpenetration*4).." pene, "..GLOCK_sync_GLhit.."% Hit"
	
		GLOCKVCHAT_Text1:SetText(GLOCK_Config["VCHAT1"])	
		GLOCKVCHAT_Text2:SetText(GLOCK_Config["VCHAT2"])
		GLOCKVCHAT_Text3:SetText(GLOCK_Config["VCHAT3"])
		GLOCKVCHAT_Text4:SetText(GLOCK_Config["VCHAT4"])


	if (GLOCK_Config["PVP"]==0 and string.find(GLOCK_sync_Name,"*")) then return end
	--Do Damage calculations
	GLOCK_ChatDmgParse()
	
	
end


-----------------------------------------------------------------------
--Resistance Calculation for data coming over the chat channel
function GLOCK_ChatDmgParse()

		if not GLOCK_sync_dbindex then return end
		if not GLOCK_sync_gmitigated then return end
	
		if not MobResistDB[GLOCK_sync_dbindex] then MobResistDB[GLOCK_sync_dbindex]="0/0_0A0|0"; end

		a=MobResistDB[GLOCK_sync_dbindex];
		GLOCK_temp1=string.find(a,"/");
		GLOCK_temp2=string.find(a,"_");
		GLOCK_temp3=string.find(a,"A");
		GLOCK_temp4=string.find(a,"|");
		
		GLOCK_DBMiti=tonumber(string.sub(a,1,GLOCK_temp1-1));				--Current Approximation
		GLOCK_DBHits=tonumber(string.sub(a,GLOCK_temp1+1,GLOCK_temp2-1))		--#of spells that have hit
		GLOCK_DBMiss=tonumber(string.sub(a,GLOCK_temp2+1,GLOCK_temp3-1))		--#of 100% resists
		GLOCK_DBMinR=tonumber(string.sub(a,GLOCK_temp3+1,GLOCK_temp4-1))		--Minimum spell penetration that a partial resist has been observed at
		GLOCK_DBi=tonumber(string.sub(a,GLOCK_temp4+1))					--Hit number for first partial resist to start learning

	if GLOCK_sync_gmitigated==100 then 
		--Average Penetration, Average Hit%, Miss increment
		GLOCK_DBMiss=GLOCK_DBMiss+1;
		GLOCK_DBi=(GLOCK_DBi*(GLOCK_DBHits+GLOCK_DBMiss-1)+(GLOCK_sync_GLhit))/(GLOCK_DBHits+GLOCK_DBMiss)

		if GLOCK_sync_school==GLOCK_schools[3] then
			GLHITS=GLOCK_DBMiss+GLOCK_DBHits;
			GLOCK_temp1=GLhit;
			GLhit=GLOCK_sync_GLhit;
			GLOCK_DBMiti=100-((100-GLOCK_DBMiti)*(GLHITS-1)+(0-GLOCK_sync_gpenetration)/GLOCK_levelMiss(GLOCK_sync_Level))/GLHITS
			GLhit=GLOCK_temp1;
			GLOCK_DBMinR=GLOCK_DBMinR-(GLOCK_DBMinR-GLOCK_sync_gpenetration*4)*GLOCK_factor
		end

		MobResistDB[GLOCK_sync_dbindex]=string.format("%f/%f_%fA%f|%f",GLOCK_DBMiti,GLOCK_DBHits,GLOCK_DBMiss,GLOCK_DBMinR,GLOCK_DBi);
		if GLOCK_Target_Name==GLOCK_sync_Name then GLOCK_UpdateGUI() end;

		return
	end
	
	if GLOCK_sync_gpenetration==nil then return end

--Average Penetration, Average Hit%, Hit increment
	GLOCK_DBHits=GLOCK_DBHits+1;


--GLOCK_factor adapts as you get closer to the target (i.e. more samples available)
	if (GLOCK_DBHits)>GLCUSPS[1] then GLOCK_factor=GLFACTORS[1] end
	if (GLOCK_DBHits)>GLCUSPS[2] then GLOCK_factor=GLFACTORS[2] end
	if (GLOCK_DBHits)>GLCUSPS[3] then GLOCK_factor=GLFACTORS[3] end
	if (GLOCK_DBHits)>GLCUSPS[4] then GLOCK_factor=GLFACTORS[4] end
	if (GLOCK_DBHits)>GLCUSPS[5] then GLOCK_factor=GLFACTORS[5] end
	if (GLOCK_DBHits)>GLCUSPS[6] then GLOCK_factor=GLFACTORS[6] end
	if (GLOCK_DBHits)>GLCUSPS[7] then GLOCK_factor=GLFACTORS[7] end
	if (GLOCK_DBHits)>GLCUSPS[8] then GLOCK_factor=GLFACTORS[8] end

	GLOCK_DBi=(GLOCK_DBi*(GLOCK_DBHits+GLOCK_DBMiss-1)+(GLOCK_sync_GLhit))/(GLOCK_DBHits+GLOCK_DBMiss)
	GLOCK_DBMinR=GLOCK_DBMinR-(GLOCK_DBMinR-GLOCK_sync_gpenetration*4)*GLOCK_factor
	
--CALCULATE THE NEW RESIST VALUE
	if GLOCK_sync_school~=GLOCK_schools[3] then
		GLOCK_diff=(GLOCK_sync_gmitigated+GLOCK_sync_gpenetration-GLOCK_DBMiti)*GLOCK_factor;
		GLOCK_DBMiti=GLOCK_DBMiti + GLOCK_diff;
	end

--SPECIFIC CALCULATION FOR BINARY FROST SPELLS
	if GLOCK_sync_school==GLOCK_schools[3] then
		GLHITS=GLOCK_DBMiss+GLOCK_DBHits;
		GLOCK_temp1=GLhit;
		GLhit=GLOCK_sync_GLhit;
		GLOCK_DBMiti=100-((100-GLOCK_DBMiti)*(GLHITS-1)+(100-GLOCK_sync_gmitigated-GLOCK_sync_gpenetration)/GLOCK_levelMiss(GLOCK_sync_Level))/GLHITS
		GLhit=GLOCK_temp1;
	end

--STORE DATA
	MobResistDB[GLOCK_sync_dbindex]=string.format("%f/%f_%fA%f|%f",GLOCK_DBMiti,GLOCK_DBHits,GLOCK_DBMiss,GLOCK_DBMinR,GLOCK_DBi);

--UPDATE GUI
	GLOCK_sync_Name=gsub(GLOCK_sync_Name,"*","")
	if UnitName("Target")==GLOCK_sync_Name then GLOCK_UpdateGUI() end;
	
	

end

---------------------------------------------------------------
-- This mod is a smart curse application to make sure that CoS, CoE and CoR are up on the mob in that order.
function GLOCK_raidCurse()

if UnitName("target")==nil then return; end
if UnitIsPlayer("target") then CastSpellByName(GL_COSr); return; end

GLc0s=0; GLc0e=0; GLc0r=0;
for i=1,16 do
	GLTooltip:ClearLines();	GLTooltip:SetUnitDebuff("Target",i);
	if (GLTooltipTextLeft1:GetText()==GL_COS) then GLc0s=1; end
	if (GLTooltipTextLeft1:GetText()==GL_COE) then GLc0e=1; end
	if (GLTooltipTextLeft1:GetText()==GL_COR) then GLC0R=1; end
end

if GLc0s==0 then CastSpellByName(GL_COSr)
elseif (GLc0s==1 and GLc0e==0) then CastSpellByName(GL_COEr)
elseif (GLc0s==1 and GLc0e==1 and GLc0r==0) then CastSpellByName(GL_CORr)
elseif (GLc0s==1 and GLc0e==1 and GLc0r==1) then CastSpellByName(GL_COAr) end

end


function gljoin(GChanName)
	--CONNECT TO THE DEFAULT CHANNEL
	if (GLOCK_Config["CHATCHANNEL"]~=GChanName and GLOCK_Config["CHATCHANNEL"]~=nil) then LeaveChannelByName(GLOCK_Config["CHATCHANNEL"]); DEFAULT_CHAT_FRAME:AddMessage("<GLOCK> Left: "..GLOCK_Config["CHATCHANNEL"])end
	GLOCK_Config["CHATCHANNEL"]=GChanName
	GLOCK_Config["CHATCHANNEL"]=string.gsub(GLOCK_Config["CHATCHANNEL"]," ","")
	JoinChannelByName(GLOCK_Config["CHATCHANNEL"]);
	DEFAULT_CHAT_FRAME:AddMessage("<GLOCK> Joined Link Channel: "..GLOCK_Config["CHATCHANNEL"])
	GLTooltip:RegisterEvent("CHAT_MSG_CHANNEL");	
end

function GLOCK_syncChanRaid()
	if string.find(arg1,"<CTMod>") and string.find(arg1,"Channel changed to:") then
		GLOCK_temp1=string.find(arg1,"to:");
		gljoin(string.sub(arg1,GLOCK_temp1+4))
	end
end


----------------------------------------------------------------
--This function determines the school of magic on a total resist (it doesn't explicity say "for x fire damage", for example)
function GLOCK_DecipherSchool(a)

	GLOCK_cast_school=nil;

	for i=1,table.getn(GLOCK_Fire) do if string.find(a,GLOCK_Fire[i]) then GLOCK_cast_school=GLOCK_schools[2]; end end
	for i=1,table.getn(GLOCK_Shadow) do if string.find(a,GLOCK_Shadow[i]) then GLOCK_cast_school=GLOCK_schools[5]; end end
	for i=1,table.getn(GLOCK_Arcane) do if string.find(a,GLOCK_Arcane[i]) then GLOCK_cast_school=GLOCK_schools[1]; end end
	for i=1,table.getn(GLOCK_Nature) do if string.find(a,GLOCK_Nature[i]) then GLOCK_cast_school=GLOCK_schools[4]; end end
	for i=1,table.getn(GLOCK_Frost) do if string.find(a,GLOCK_Frost[i]) then GLOCK_cast_school=GLOCK_schools[3]; end end

--DETECT WAND SCHOOL
	if string.find(a,GLWANDACTION) then

		--Wand slot is 18
		GLTooltip:SetInventoryItem("Player", 18);
		itemName = GLTooltipTextLeft1:GetText();
		lines = GLTooltip:NumLines();		
			for i=2, lines do
				tmpText = getglobal("GLTooltipTextLeft"..i);
				tmpStr=tmpText:GetText()
				if (tmpStr) then
					if string.find(tmpStr,GL_WAND_TEXT[1]) then GLOCK_cast_school=GLOCK_schools[1]; end
					if string.find(tmpStr,GL_WAND_TEXT[2]) then GLOCK_cast_school=GLOCK_schools[2]; end
					if string.find(tmpStr,GL_WAND_TEXT[3]) then GLOCK_cast_school=GLOCK_schools[3]; end
					if string.find(tmpStr,GL_WAND_TEXT[4]) then GLOCK_cast_school=GLOCK_schools[4]; end
					if string.find(tmpStr,GL_WAND_TEXT[5]) then GLOCK_cast_school=GLOCK_schools[5]; end
				end
					
			end
	end
	
end

----------------------------------------------------------------------------------------------------------
function GLOCK_DBSearch()
jj=1
a=GLOCK_Name_InputBox:GetText()
b=GLOCK_Level_InputBox:GetText()
GLOCK_SearchTemp={}

	for index,value in pairs(MobResistDB) do
		GLOCK_temp1=string.find(index,"/");
		GLOCK_tmpStr="";
		if GLOCK_temp1 then
			GLOCK_temp2=string.find(index,"/",GLOCK_temp1+1)
			GLOCK_temp3=string.sub(index,1,GLOCK_temp1-1)
			GLOCK_tmpStr=string.sub(index,1,GLOCK_temp2-1)
		end
		if (string.find(string.lower(GLOCK_tmpStr),string.lower(a)) and string.find(string.lower(GLOCK_tmpStr),string.lower(b))) then 
			GLOCK_temp1=string.find(index,"/");
			if GLOCK_temp1 then
				GLOCK_temp2=string.find(index,"/",GLOCK_temp1+1)

				GLOCK_temp3=string.sub(index,1,GLOCK_temp1-1)
				GLOCK_temp4=tonumber(string.sub(index,GLOCK_temp1+1,GLOCK_temp2-1))
				if GLOCK_temp4==-1 then GLOCK_temp4="Boss" end
					
				if jj>9 then return end

				GLOCK_temp1=0;
				for indexi, valuei in ipairs(GLOCK_SearchTemp) do if string.sub(index,1,GLOCK_temp2-1)==valuei then GLOCK_temp1=1; end end
				
				if GLOCK_temp1==0 then
					getglobal("GLOCKSearch_B"..jj):Show()
					getglobal("GLOCKSearch_Text"..jj):SetText(GLOCK_temp4.." - "..GLOCK_temp3)
					GLOCK_SearchTemp[jj]=string.sub(index,1,GLOCK_temp2-1)
					jj=jj+1
				end
			end
		 end
		
	end
	if jj<9 then for i=jj,9 do getglobal("GLOCKSearch_Text"..i):SetText("") getglobal("GLOCKSearch_B"..i):Hide() end end

end

--------------------------------------------------------------------------------------------
function GLOCK_DBSearchDisplay(a)
		GLOCK_frame:Show()
		GLOCK_minimize:Hide()
		ClearTarget()

		GLOCK_temp1=string.find(GLOCK_SearchTemp[a],"/");
		GLOCK_S_Name=string.sub(GLOCK_SearchTemp[a],1,GLOCK_temp1-1)
		GLOCK_S_Level=tonumber(string.sub(GLOCK_SearchTemp[a],GLOCK_temp1+1))

		GLOCK_Target_Name=GLOCK_S_Name
		GLOCK_Target_Level=GLOCK_S_Level

	jj=1;

	for i=1,5 do
		GLOCK_dbindex=GLOCK_S_Name.."/"..GLOCK_S_Level.."/"..GLOCK_schools[i]
		getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):Hide()
		getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):Hide()

		if (MobResistDB[GLOCK_dbindex.."immune"]==1 or MobResistDB[GLOCK_dbindex.."vuln"]==1) then 
			if MobResistDB[GLOCK_dbindex.."immune"]==1 then GLSpecial="Imm" end
			if MobResistDB[GLOCK_dbindex.."vuln"]==1 then GLSpecial="Vul" end
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetText(GLSpecial) 
			getglobal("GMin"..GLOCK_UIschools[i].."TxT"):SetText(GLSpecial)
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetText("0")
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetText("0")

			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):Show()

			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):SetPoint("TOPLEFT",5,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):SetPoint("TOPLEFT",25,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetPoint("TOPLEFT",70,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetPoint("TOPLEFT",110,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetPoint("TOPLEFT",160,-45-10*(jj-1));
			jj=jj+1;			
		end

		if MobResistDB[GLOCK_dbindex]~=nil and (not (MobResistDB[GLOCK_dbindex.."immune"]==1 or MobResistDB[GLOCK_dbindex.."vuln"]==1)) then 
			a=MobResistDB[GLOCK_dbindex];
			GLOCK_temp1=string.find(a,"/");
			GLOCK_temp2=string.find(a,"_");
			GLOCK_temp3=string.find(a,"A");
			GLOCK_temp4=string.find(a,"|");
			
			GLOCK_DBMiti=tonumber(string.sub(a,1,GLOCK_temp1-1));			
			GLOCK_DBHits=tonumber(string.sub(a,GLOCK_temp1+1,GLOCK_temp2-1))
			GLOCK_DBMiss=tonumber(string.sub(a,GLOCK_temp2+1,GLOCK_temp3-1))	
			GLOCK_DBMinR=tonumber(string.sub(a,GLOCK_temp3+1,GLOCK_temp4-1))	
			GLOCK_DBi=tonumber(string.sub(a,GLOCK_temp4+1))				
			if GLOCK_DBMiti<0 then GLOCK_DBMiti=0 end
			
		--CHECK FOR ERRORS
			if GLOCK_DBMiti==nil then GLOCK_DBMiti=0; MobResistDB[GLOCK_dbindex]=nil; return end
			if GLOCK_DBHits==nil then GLOCK_DBHits=0; MobResistDB[GLOCK_dbindex]=nil; return end
			if GLOCK_DBMiss==nil then GLOCK_DBMiss=0; MobResistDB[GLOCK_dbindex]=nil; return end
			
		--Update Resist Approximations
			GTxT=tostring(GLOCK_round(4*GLOCK_DBMiti));
			getglobal("GMin"..GLOCK_UIschools[i].."TxT"):SetText(GTxT)
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetText(GTxT)
			
		--Display #Casts
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetText(GLOCK_round(GLOCK_DBHits+GLOCK_DBMiss))

		--Display %Hit (avg +hit%)  (i.e. 86(3)% )
			GTxT=string.format("%.0f(%.0f)",GLOCK_round(100*GLOCK_DBHits/(GLOCK_DBMiss+GLOCK_DBHits)),GLOCK_DBi)
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetText(GTxT)

		--Display Current Penetration (avg Penetration) (i.e. 75(60) )
			GTxT=string.format("(%.0f)",GLOCK_round(GLOCK_DBMinR))
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):SetText(GTxT)


			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):Show()
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):Show()

			getglobal("GLOCK_"..GLOCK_UIschools[i].."VulnT"):SetPoint("TOPLEFT",5,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."Vuln"):SetPoint("TOPLEFT",25,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."H"):SetPoint("TOPLEFT",70,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."M"):SetPoint("TOPLEFT",110,-45-10*(jj-1));
			getglobal("GLOCK_"..GLOCK_UIschools[i].."R"):SetPoint("TOPLEFT",160,-45-10*(jj-1));

			jj=jj+1;
		elseif  ((not MobResistDB[GLOCK_dbindex]) and (not MobResistDB[GLOCK_dbindex.."immune"]) and (not MobResistDB[GLOCK_dbindex.."vuln"])) then
			getglobal("GMin"..GLOCK_UIschools[i].."TxT"):SetText("")
		end
	end

		GLOCK_Tlvl:SetText(GLOCK_S_Level)
		GLOCK_TName:SetText(GLOCK_S_Name)

		if GLOCK_S_Level>=0 then GLOCK_Tlvl:SetTextColor(.5,.5,.5) end
		if GLOCK_S_Level>=48 then GLOCK_Tlvl:SetTextColor(.2,.9,.2) end
		if (GLOCK_S_Level>=58 and GLOCK_Target_Level<=62) then GLOCK_Tlvl:SetTextColor(1,1,0) end
		if (GLOCK_S_Level==63) then GLOCK_Tlvl:SetTextColor(.9,.5,.4) end
		if (GLOCK_S_Level>63) then GLOCK_Tlvl:SetTextColor(1,.2,.2) end
		if (GLOCK_S_Level==-1) then GLOCK_Tlvl:SetTextColor(1,.2,.2); GLOCK_Tlvl:SetText("Boss") end

		GLOCK_frame:Show()
		GLOCK_minimize:Hide()
end

----------------------
--Set the global player config
function GLOCK_Initialize()

	GLFACTORS={.2, .05, .04, .03, .02, .01, .008, .005}
	GLCUSPS={0,30,40,60,100,150,200,500}
	GLOCK_factor=GLFACTORS[1]

	GLOCK_MINTIP="";
	GLOCK_TIME=0;


	GLOCK_UIschools={"Arcane","Fire","Frost","Nature","Shadow"};	--FOR UI COMPONENT INDEXING (must stay in english)


	GLOCK_frame:Hide();
	GLOCK_frame:SetHeight(50+12*5)
	GLOCK_frame:SetWidth(200)
		
	if not MobResistDB then
		MobResistDB	= {	}
	end

	
	for i=1,9 do getglobal("GLOCKSearch_B"..i):Hide() end

	-- Add Slash Commands

	SlashCmdList["GLRAIDCURSE"] = GLOCK_raidCurse;
	SLASH_GLRAIDCURSE1 = "/glraidcurse";

	SlashCmdList["GLOCKVER"] = GLOCK_version;
	SLASH_GLOCKVER1 = "/glockver";
	
	SlashCmdList["GLJOIN"] = gljoin;
	SLASH_GLJOIN1 = "/gljoin";

	SlashCmdList["GLOCKHELP"] = GLOCK_help;
	SLASH_GLOCKHELP1 = "/glockhelp";

	SlashCmdList["GLOCK"] = glconfig;
	SLASH_GLOCK1 = "/glock";

	SlashCmdList["GLTEST"] = GLOCK_test;
	SLASH_GLTEST1 = "/gltest";

	SlashCmdList["GLOCKLOCK"] = GLOCK_lock;
	SLASH_GLOCKLOCK1 = "/glocklock";

	GLElemPrec=0
	GLHit=0;
	GLDshadow=0;
	GLDarcane=0;
	GLDfrost=0;
	GLDfire=0;
	GLDnature=0;

	GLOCK_Target_Level=0
	GLOCK_Target_Name=""

		GLOCK_frame:Hide()
		GLOCK_minimize:Hide()
		GLOCK_confirm:Hide()

	if not GLOCK_Config then
		GLOCK_Config={ }
		GLOCK_Config["VISIBLE"]=1
		GLOCK_Config["PVP"]=0
		GLOCK_Config["AUTOLINK"]=0
		GLOCK_Config["SEARCHON"]=1
		GLOCK_Config["LOCKED"]=0
	end
		GLOCK_Config["VCHAT1"]=""
		GLOCK_Config["VCHAT2"]=""
		GLOCK_Config["VCHAT3"]=""
		GLOCK_Config["VCHAT4"]=""

	if GLOCK_Config["LOCKED"]==1 then GLOCK_frame:SetScript("OnMouseDown",nil) GLOCK_minimize:SetScript("OnMouseDown",nil) end

	GLOCK_config_Visible_CheckBox:SetChecked(GLOCK_Config["VISIBLE"])
	GLOCK_config_pvp_CheckBox:SetChecked(GLOCK_Config["PVP"])
	GLOCK_config_LinkOnLoad_CheckBox:SetChecked(GLOCK_Config["AUTOLINK"])
	
	GLOCK_Level_InputBox:Hide()
	GLOCK_Name_InputBox:Hide()
	GLOCK_config_SearchName_Text:Hide()
	GLOCK_config_SearchLevel_Text:Hide()
	GLOCKSearch_Button:Hide()
	GLOCKSearch_ScrollFrame:Hide()
	for i=1,9 do getglobal("GLOCKSearch_B"..i):Hide() end
	for i=1,9 do getglobal("GLOCKSearch_Text"..i):SetText("") end


	this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("UNIT_AURASTATE");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("CHAT_MSG_RAID");


	if GLOCK_Config["CHATCHANNEL"] and GLOCK_Config["AUTOLINK"] then gljoin(GLOCK_Config["CHATCHANNEL"]) end
	GLOCK_scanGear()
	GLOCK_version()
end

function GLOCK_round(x)

a=x-math.floor(x);
if a>=.5 then y=math.ceil(x) else y=math.floor(x) end
return y

end